#
# Copyright (C) 2021 Associated Universities, Inc. Washington DC, USA.
#
# This file is part of NRAO Workspaces.
#
# Workspaces is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Workspaces is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Workspaces.  If not, see <https://www.gnu.org/licenses/>.
import logging
from typing import Dict

import chevron
import sqlalchemy as sa
from sqlalchemy.ext.declarative import declarative_base

from workspaces.system.schema import AbstractFile

logger = logging.getLogger(__name__)

Base = declarative_base()


class NotificationTemplate(Base):
    """
    Notification Templates are Mustache-formatted files used for sending emails with the notification system.
    """

    __tablename__ = "notification_templates"
    name = sa.Column("name", sa.String, primary_key=True)
    description = sa.Column("description", sa.String, nullable=False)
    content = sa.Column("template", sa.String, nullable=False)

    def render(self, argument: Dict) -> AbstractFile:
        """
        Render a given Mustache-formatted Notifiation Template with the specified arguments.

        :param argument:    argument(s) needed for template render
        :return:            an AbstractFile rendered from mustache template
        """
        contents = chevron.render(self.content, argument)
        logger.info(
            "rendering %s with argument %s to %s",
            self.content,
            argument,
            contents,
        )
        return AbstractFile(self.name, contents)

    def __json__(self, request):
        return dict(name=self.name, description=self.description, content=self.content)
